# NanoBlogger Picture Gallery plugin
#
# Copyright 2005, Niels Heirbaut <niels@heirbaut.nl>
# This program is released under the GPL v2.

# How it works:
# Place all pictures for the gallery in a subdir of $BLOG_DIR/images/galleries
# the plugin will pick up the pictures from there an create the gallery.
# An entry called intro.txt can be used as introduction to the gallery. Just
# generate it using nb --draft intro.txt in you gallery subdir.
#
# If you are using a version of NanoBlogger that does not have the --draft
# option (<3.3) you can also hand craft one. The simplest method is to copy one
# of your entries from $BLOG_DIR/data to the gallery directory, rename it to
# 'intro.txt' and replace ALL entries in there (especially the date!)
#
# A file named annotations.txt can be used to put annotations with selected
# pictures. The format is:
#
#   <picture_file>,<annotation>

# sample code for templates, based off the default stylesheet
#
# <div class="side">
# $NB_GalleryLinks
# </div>

GALLERY_PLUGIN_OUTFILE="$BLOG_DIR/$PARTS_DIR/gallery_links.$NB_FILETYPE"
> "$GALLERY_PLUGIN_OUTFILE"

set_gallery_baseurl(){
    OLD_BASE_URL=$BASE_URL
    GALLERY_BASEDIR=${GALLERY_BASEDIR%/}
    BASE_URL="../"

    OFS=$IFS; IFS='/'
    for d in $GALLERY_BASEDIR ; do
        BASE_URL="../$BASE_URL"
    done
    IFS=$OFS
}

set_galleryconf(){
    : ${GALLERY_BASEDIR:=images/galleries}
    : ${GALLERY_TEMPLATE:=$NB_TEMPLATE_DIR/$MAKEPAGE_TEMPLATE}
    : ${GALLERY_THUMBNAILCMD:=convert}
    : ${GALLERY_ANNOTATIONFILE:=annotations.txt}
    : ${GALLERY_THUMBNAILSUBDIR:=thumbs}
    : ${GALLERY_THUMBNAILSIZE:=150}
    : ${GALLERY_INTROFILE:=intro.txt}
    : ${GALLERY_ROWITEMS:=4}

    set_gallery_baseurl
    NBVERSION="${VERSION%% *}"
}

reset_galleryconf(){
    GALLERY_TEMPLATE=
    set_galleryconf
}

# Compares version numbers of the format "x[.y]*", where x and y are integer
# numbers.
# Returns  0 if n1 == n2
#          1 if n1 < n2
#          2 if n1 > n2
compare_versions(){
    [ "x$1" == "x$2" ] && return 0

    # Split the version number elements. Strip everything from the version
    # number that is not conforming to the "x[.y]*" pattern
    local ofs=$IFS
    IFS='.'
    local -a v1=( ${1%%[^0-9\.]*} )
    local -a v2=( ${2%%[^0-9\.]*} )
    IFS=$ofs

    # Determine the maximum number of elements
    local n=${#v1[@]}
    [ $n -lt ${#v2[@]} ] && n=${#v2[@]}

    # Compare all possible elements
    for ((i=0; i < n; i++)); do
        [ -z "${v1[i]}" ] && v1[$i]=0
        [ -z "${v2[i]}" ] && v2[$i]=0
        [ ${v1[i]} -lt ${v2[i]} ] && return 1
        [ ${v1[i]} -gt ${v2[i]} ] && return 2
    done

    return 0
}

add_gallerylink(){
    echo '<a href="'$GALLERY_BASEDIR/`basename $GALLERY_DIR`/index.html'">'$GALLERY_TITLE'</a><br />' >> "$GALLERY_PLUGIN_OUTFILE"
}

create_thumbnail(){
    [ ! -d "$GALLERY_THUMBNAILDIR" ] && mkdir "$GALLERY_THUMBNAILDIR"

    if [ ! -f $GALLERY_THUMBNAILDIR/$PICTURE ] ; then
        nb_msg "generating thumbnail for $PICTURE ..."
        $GALLERY_THUMBNAILCMD "$GALLERY_DIR/$PICTURE" -resize $GALLERY_THUMBNAILSIZE "$GALLERY_THUMBNAILDIR/$PICTURE"
    fi
}

set_pagetitle(){
    METADATA=
    read_metadata TITLE "$GALLERY_DIR/$GALLERY_INTROFILE"

    compare_versions "$VERSION" "3.2"

    [ $? -eq 1 ] && METADATA=$NB_Metadata

    if [ -z "$METADATA" ]; then
        GALLERY_TITLE="`basename $GALLERY_DIR`"
    else
        GALLERY_TITLE="$METADATA"
    fi
}

put_annotation(){
    annotation=
    if [ -f $GALLERY_DIR/$GALLERY_ANNOTATIONFILE ] ; then
        annotation=$(grep $PICTURE $GALLERY_DIR/$GALLERY_ANNOTATIONFILE | sed -e "s/^$PICTURE,\(.*\)$/\1/")
        if [ ! -z "$annotation" ] ; then
            echo "<p><div class=\"annotation\">$annotation</div></p>" >> $GALLERY_OUTFILE
        fi
    fi
}

put_introtext(){
    compare_versions "$VERSION" "3.3"
    local versions_compared=$?

    if [ -f $GALLERY_DIR/$GALLERY_INTROFILE ] ; then
        if [ $versions_compared -eq 1 ] ; then
            read_metadata BODY "$GALLERY_DIR/$GALLERY_INTROFILE"
            NB_EntryBody=$NB_Metadata
        else
            read_metadata "BODY,$METADATA_CLOSETAG" $GALLERY_DIR/$GALLERY_INTROFILE
            NB_EntryBody=$METADATA
        fi

        if [ $versions_compared -ne 1 ] ; then
            read_metadata FORMAT $GALLERY_DIR/$GALLERY_INTROFILE
            load_plugins entry/format "$METADATA"
        fi

        if [ ! -z "$NB_EntryBody" ] ; then
            cat >> "$GALLERY_OUTFILE" <<-EOF
                <td colspan=$GALLERY_ROWITEMS><div id="introtext">$NB_EntryBody</div></td>
                </tr><tr>
EOF
        fi
    fi
}

put_thumbnail()
{
    create_thumbnail
    cat >> "$GALLERY_OUTFILE" <<-EOF
        <td width=$CELLWIDTH%><div class="picture"><p>
        <a href="$GALLERY_URL/$PICTURE">
            <img alt="$PICTURE" src="$GALLERY_URL/$GALLERY_THUMBNAILSUBDIR/$PICTURE" />
        </a></p></div>
EOF
}

gallery_remove_macmeta(){
    for metapicture in `find $GALLERY_DIR -maxdepth 1 -name ._\*.jpg -o -name ._\*.JPG -o -name ._\*.png -o -name ._\*.PNG`
    do
        rm -f $metapicture
    done
}

create_gallery(){
    GALLERY_OUTFILE="$GALLERY_DIR/tmp.html"
    GALLERY_PAGEFILE="$GALLERY_DIR/index.html"
    GALLERY_URL="${BASE_URL}$GALLERY_BASEDIR/`basename $GALLERY_DIR`"
    GALLERY_THUMBNAILDIR="$GALLERY_DIR/$GALLERY_THUMBNAILSUBDIR"
    CELLWIDTH=$((100 / $GALLERY_ROWITEMS))

    cat > "$GALLERY_OUTFILE" <<-EOF
        <div class="gallery">
        <table summary="NanoBlogger gallery">
        <tr>
EOF

    put_introtext

    count=0
    for PICTURE in `find $GALLERY_DIR -maxdepth 1 -name \*.jpg -o -name \*.JPG -o -name \*.png -o -name \*.PNG`
    do
        PICTURE=`basename $PICTURE`
        if [ $((count % $GALLERY_ROWITEMS )) -eq 0 ]; then
            if [ $count -ne 0 ] ; then
                echo "</tr><tr>" >> $GALLERY_OUTFILE
            fi
        fi
        put_thumbnail
        put_annotation
        echo "</td>" >> $GALLERY_OUTFILE
        count=$((count+1))
    done

    cat >> "$GALLERY_OUTFILE" <<-EOF
        </tr>
        </table>
        </div>
EOF
}

set_galleryconf

if [ -d "$BLOG_DIR/$GALLERY_BASEDIR" ]; then
    nb_msg "generating picture galleries ..."
    for GALLERY_DIR in `find $BLOG_DIR/$GALLERY_BASEDIR -mindepth 1 -maxdepth 1 -type d`; do
        nb_msg $GALLERY_DIR
        GALLERY_TITLE=
        create_gallery
        set_pagetitle
        if [ -e $GALLERY_OUTFILE ]; then
            MKPAGE_TITLE="$GALLERY_TITLE"
            make_page "$GALLERY_OUTFILE" "$GALLERY_TEMPLATE" "$GALLERY_PAGEFILE"
            rm -f $GALLERY_OUTFILE
        fi
        add_gallerylink

        [ $(uname) = 'Darwin' ] && gallery_remove_macmeta

    done
fi

NB_GalleryLinks=$(< "$GALLERY_PLUGIN_OUTFILE")

reset_galleryconf

BASE_URL=$OLD_BASE_URL
